<?php
/**
 * The create ONLINEmaps settings page.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps;
use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps\Locations\Main as LocationsMain;
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\{OnlineMaps as OnlineMapsRepository, Styles as StylesRepository};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\{BorderType, Height, Id as OnlineMapId, Name, Style, Width};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for rendering the main online maps settings page.
 */
final class Edit {
	public const NONCE_ACTION = 'edit_onlinemap';

	/**
	 * This function processes the form.
	 *
	 * @param OnlineMap $onlinemap The ONLINEmap.
	 */
	private static function try_process_form_action( OnlineMap $onlinemap ): void {
		$request_method = filter_input( INPUT_SERVER, 'REQUEST_METHOD', FILTER_DEFAULT ) ?? '';
		if ( empty( $request_method ) && isset( $_SERVER['REQUEST_METHOD'] ) ) {
			$request_method = filter_var( wp_unslash( $_SERVER['REQUEST_METHOD'] ), FILTER_DEFAULT );
		}
		if ( 'POST' !== $request_method ) {
			return;
		}

		$admin_url = add_query_arg( 'page', OnlineMaps::SUB_PAGE_NAME, admin_url( 'admin.php' ) );

		$wp_nonce = (string) ( filter_input( INPUT_POST, '_wpnonce', FILTER_DEFAULT ) ?? '' );
		if ( ! wp_verify_nonce( $wp_nonce, self::NONCE_ACTION ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$name = filter_input( INPUT_POST, 'name', FILTER_DEFAULT );
		if ( is_string( $name ) ) {
			$onlinemap->update_name( new Name( $name ) );
		}

		$style = filter_input( INPUT_POST, 'style', FILTER_DEFAULT );
		if ( is_string( $style ) ) {
			$onlinemap->update_style( new Style( $style ) );
		}
		$width = filter_input( INPUT_POST, 'width', FILTER_DEFAULT );
		if ( is_string( $width ) ) {
			$onlinemap->update_width( new Width( $width ) );
		}
		$height = filter_input( INPUT_POST, 'height', FILTER_DEFAULT );
		if ( is_string( $height ) ) {
			$onlinemap->update_height( new Height( $height ) );
		}
		$border_type = filter_input( INPUT_POST, 'border-type', FILTER_DEFAULT );
		if ( is_string( $border_type ) ) {
			$onlinemap->update_border_type( BorderType::tryFrom( $border_type ) ?? BorderType::None );
		}

		$onlinemaps_repository = new OnlineMapsRepository();
		try {
			$onlinemaps_repository->update( $onlinemap );

			print '<div class="updated">';
			print '<p>' . esc_html__( 'ONLINEmap updated!', 'locrmaps-textdomain' ) . '</p>';
			print '<p><a href="' . esc_url( $admin_url ) . '">← ' . esc_html__( 'Back to the ONLINEmaps list', 'locrmaps-textdomain' ) . '</a></p>';
			print '</div>' . PHP_EOL;
		} catch ( \Exception $exc ) {
			print '<div class="error"><p>' . esc_html__( 'ONLINEmap not updated!', 'locrmaps-textdomain' ) . '</p></div>' . PHP_EOL;
		}
	}

	/**
	 * This function renders the main online maps settings page.
	 */
	public static function render(): void {
		print '<div class="wrap">' . PHP_EOL;
		print '<h1>' . esc_html__( 'locrMAPS', 'locrmaps-textdomain' ) . ' &rsaquo; ' . esc_html__( 'ONLINEmaps', 'default' ) . '</h1>' . PHP_EOL;

		$admin_url = add_query_arg( 'page', OnlineMaps::SUB_PAGE_NAME, admin_url( 'admin.php' ) );
		$wp_nonce  = (string) ( filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? '' );
		if ( ! wp_verify_nonce( $wp_nonce, self::NONCE_ACTION ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$id                    = intval( filter_input( INPUT_GET, 'id', FILTER_VALIDATE_INT ) ?? 0 );
		$onlinemaps_repository = new OnlineMapsRepository();
		$onlinemap             = $onlinemaps_repository->get_by_id( new OnlineMapId( $id ) );
		if ( is_null( $onlinemap ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapNotFound->value, $admin_url ) );
			exit;
		}

		$tab    = (string) ( filter_input( INPUT_GET, 'tab', FILTER_DEFAULT ) ?? '' );
		$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );

		$admin_url = add_query_arg(
			array(
				'action'   => $action,
				'id'       => $id,
				'_wpnonce' => $wp_nonce,
			),
			$admin_url
		);

		$general_tab_classes   = array( 'nav-tab' );
		$locations_tab_classes = array( 'nav-tab' );
		match ( $tab ) {
			'locations' => $locations_tab_classes[] = 'nav-tab-active',
			default     => $general_tab_classes[]   = 'nav-tab-active',
		};

		print '<nav class="nav-tab-wrapper">' . PHP_EOL;
		print '	<a href="' . esc_attr( add_query_arg( 'tab', 'general', $admin_url ) ) . '" class="' . esc_attr( implode( ' ', $general_tab_classes ) ) . '">' . esc_html__( 'General', 'default' ) . '</a>' . PHP_EOL;
		print '	<a href="' . esc_attr( add_query_arg( 'tab', 'locations', $admin_url ) ) . '" class="' . esc_attr( implode( ' ', $locations_tab_classes ) ) . '">' . esc_html__( 'Locations', 'locrmaps-textdomain' ) . '</a>' . PHP_EOL;
		print '</nav>' . PHP_EOL;

		print '<br />';

		match ( $tab ) {
			'locations' => LocationsMain::process_and_render_locations_form( $onlinemap->id ),
			default     => self::process_and_render_onlinemap_form( $onlinemap ),
		};

		print '</div>' . PHP_EOL;
	}

	/**
	 * This function renders the general tab.
	 *
	 * @param OnlineMap $onlinemap The online map.
	 */
	private static function process_and_render_onlinemap_form( OnlineMap $onlinemap ): void {
		self::try_process_form_action( $onlinemap );

		print '<form method="post">';
		print '	<input type="hidden" class="regular-text" id="id" name="id" readonly="readonly" value="' . esc_attr( (string) $onlinemap->id->value ) . '"/>';

		wp_nonce_field( self::NONCE_ACTION );

		print '	<table class="form-table">';
		print '		<tbody>';
		print '			<tr>';
		print '				<td><label for="shortcode">' . esc_html__( 'Shortcode', 'default' ) . '</label></td>';
		$shortcode = '[locrmaps id="' . $onlinemap->id->value . '"]';
		print '				<td><input type="text" class="regular-text" id="shortcode" name="shortcode" readonly="readonly" value="' . esc_attr( $shortcode ) . '"/></td>';
		print '			</tr>';
		print '			<tr>';
		print '				<td><label for="name">' . esc_html__( 'Name', 'locrmaps-textdomain' ) . '</label></td>';
		print '				<td><input type="text" class="regular-text" id="name" name="name" value="' . esc_attr( (string) $onlinemap->name ) . '"/></td>';
		print '			</tr>';
		print '			<tr>';
		print '				<td><label for="style">' . esc_html__( 'Style', 'locrmaps-textdomain' ) . '</label></td>';
		print '				<td>';
		print '					<select id="style" name="style" class="regular-text">';
		$style            = (string) $onlinemap->style;
		$styles           = new StylesRepository();
		$available_styles = $styles->get_all();
		foreach ( $available_styles as $available_style ) {
			$is_selected = $style === $available_style;
			print '						<option value="' . esc_attr( $available_style ) . '"';
			if ( $is_selected ) {
				print ' selected="selected"';
			}
			print '>' . esc_html( $available_style ) . '</option>' . PHP_EOL;
		}
		print '					</select>';
		print '				</td>';
		print '			</tr>';
		print '			<tr>';
		print '				<td><label for="width">' . esc_html__( 'Width', 'default' ) . '</label></td>';
		print '				<td><input type="text" class="regular-text" id="width" name="width" value="' . esc_attr( (string) $onlinemap->width ) . '" placeholder="' . esc_attr( 'default: ' . Width::DEFAULT ) . '"/></td>';
		print '			</tr>';
		print '			<tr>';
		print '				<td><label for="height">' . esc_html__( 'Height', 'default' ) . '</label></td>';
		print '				<td><input type="text" class="regular-text" id="height" name="height" value="' . esc_attr( (string) $onlinemap->height ) . '" placeholder="' . esc_attr( 'default: ' . Height::DEFAULT ) . '"/></td>';
		print '			</tr>';
		print '			<tr>' . PHP_EOL;
		print '				<td><label for="border-type">' . esc_html__( 'Border', 'locrmaps-textdomain' ) . '</label></td>' . PHP_EOL;
		print '				<td>' . PHP_EOL;
		print '					<select id="border-type" name="border-type" class="regular-text">' . PHP_EOL;
		foreach ( BorderType::cases() as $border_type ) {
			$is_selected = $onlinemap->border_type === $border_type;
			print '						<option value="' . esc_attr( $border_type->value ) . '"';
			if ( $is_selected ) {
				print ' selected="selected"';
			}
			print '>' . esc_html( $border_type->get_translated_text() ) . '</option>' . PHP_EOL;
		}
		print '					</select>' . PHP_EOL;
		print '				</td>' . PHP_EOL;
		print '			</tr>' . PHP_EOL;
		print '		</tbody>';
		print '	</table>';

		submit_button();

		print '</form>';
	}
}
